#include "intake_task.h"
#include "chassis_task.h"
#include "gimbal_task.h"
#include "detect_task.h"
#include "comm_task.h"
#include "modeswitch_task.h"
#include "info_get_task.h"
#include "info_interactive.h"
#include "infantry_info.h"
#include "judgement_info.h"
#include "remote_ctrl.h"
#include "keyboard.h"
#include "pid.h"
#include "sys_config.h"
#include "stdlib.h"
#include "math.h"
#include "string.h"
#include "cmsis_os.h"
#include "motion_profile.h"

const uint8_t BULLETS_PER_BOX = 50;

// motion profile
float lift_fast_acc = 7500, lift_slow_acc = 300, lift_fast_spd = 1250, lift_slow_spd = 300; // motion_max_acc(mm/s^2) motion_max_spd(mm/s)
float rot_fast_acc = 15000, rot_slow_acc  =600, rot_fast_spd = 15000, rot_slow_spd = 600, rot_init_acc = 300, rot_init_spd = 150; // unit in degree/s

#ifdef MASTER

  uint16_t camera_mid_pos    = (uint16_t)(1800); 
  uint16_t camera_down_pos   = (uint16_t)(1200); //! remeber to change it back
  uint16_t camera_up_pos     = (uint16_t)(1800);

#else

  uint16_t camera_mid_pos    = (uint16_t)(1800); 
  uint16_t camera_down_pos   = (uint16_t)(1200);
  uint16_t camera_up_pos     = (uint16_t)(1800);

#endif

uint8_t pc_cmd;
uint8_t pc_msg;

int32_t rot_set_pos;

UBaseType_t intake_stack_surplus;

uint8_t keep_up = 0;

intake_measure_t intake_measure_data; // measure data in intake task
intake_t 				 intake;							// mode info and PID ref&fdb 

uint16_t photoelec_delay_cnt; 			  // delay counter to make sure the photoeletric switch is obstructed
uint16_t supply_check_delay_cnt;
uint16_t close_delay_cnt;						  // delay counter to wait for paw close
uint16_t dump_delay_cnt;
uint16_t release_delay_cnt;
uint16_t limit_switch_delay_cnt;
uint16_t lift_delay_cnt;

uint8_t from_clamp_down = 0;

intake_state_e last_intake_state;

uint8_t intake_state;
uint8_t intake_place;

uint8_t supply_success, supply_fail, supply_box_sum;

float intake_angle_measure;

extern TaskHandle_t can_msg_send_task_t;

void intake_task(void const *argu)
{
	/*GPIO data read*/
	intake_measure_data.photoelectric_value       = HAL_GPIO_ReadPin(INTAKE_PHOTOELECTRIC_GPIO_Port,         INTAKE_PHOTOELECTRIC_PIN);
	intake_measure_data.supply_check_value        = HAL_GPIO_ReadPin(INTKAE_SUPPLY_GPIO_Port,                INTKAE_SUPPLY_PIN);
	intake_measure_data.clamp_reed_switch_value   = HAL_GPIO_ReadPin(INTAKE_CLAMP_REED_SWITCH_GPIO_Port,     INTAKE_CLAMP_REED_SWITCH_PIN);
	intake_measure_data.lift_limit_switch_value   = HAL_GPIO_ReadPin(INTAKE_LIFE_SWITCH_GPIO_Port,           INTAKE_LIFT_SWITCH_PIN);

	intake_angle_measure = intake_rot_get_angle();
	
	switch (intake.ctrl_mode)
	{
		case INTAKE_MANUAL:
		{	
			pc_cmd = INTAKE_CMD_VISION_LIFT_UP;
			
			if (rc.kb.bit.Z)
			{
				pc_cmd = INTAKE_CMD_LIFT_UP;
			}
			else if (rc.kb.bit.X)
			{
				pc_cmd = INTAKE_CMD_LIFT_DOWN;
			}
			else if (rc.kb.bit.C)
			{
				pc_cmd = INTAKE_CMD_CLAMP;
			}
			else
			{
				pc_cmd = INTAKE_CMD_VISION;
			}
      
      /* 
			if (!intake_measure_data.photoelectric_value)
			{
				photoelec_delay_cnt++;
				if (photoelec_delay_cnt > 100 && intake.cur_rot_state == INTAKE_NORMAL)
				{
					intake.set_rot_state = INTAKE_PICK_UP;
					photoelec_delay_cnt = 0;
        }
      }
      else
      {
        photoelec_delay_cnt = 0;
      }
      */
    }
    break;
    
    case INTAKE_AUTO:
    {
      intake_state = pc_recv_mesg.intake_control_data.intake_state;
      intake_place = pc_recv_mesg.intake_control_data.intake_place;
      
      
      switch(intake_state)
      {
        case 0:
        {
          pc_cmd = INTAKE_CMD_INIT;
        }break;
        
        case 1:
        {
          switch(intake_place)
          {
            case 0:
            {
              pc_cmd = INTAKE_CMD_VISION;
            }break;
            
            case 1:
            {
              pc_cmd = INTAKE_CMD_VISION_LIFT_UP;
            }break;
            
            case 2:
            {
              pc_cmd = INTAKE_CMD_VISION_LIFT_DOWN;
            }break;
            
            default:
            {
              pc_cmd = INTAKE_CMD_VISION;
            }break;
          }
        }break;
        
        case 2:
        {
          switch(intake_place)
          {
            case 0:
            {
              pc_cmd = INTAKE_CMD_VISION;
            }break;
            
            case 1:
            {
              pc_cmd = INTAKE_CMD_LIFT_UP;
            }break;
            
            case 2:
            {
              pc_cmd = INTAKE_CMD_LIFT_DOWN;
            }break;
            
            default:
            {
              pc_cmd = INTAKE_CMD_VISION;
            }break;
          }
        }break;
        
        case 3:
        {
          if (!intake_measure_data.photoelectric_value)
          {
            photoelec_delay_cnt++;
            if (photoelec_delay_cnt > 10)
            {
              pc_cmd = INTAKE_CMD_CLAMP;
              photoelec_delay_cnt = 0;
            }
          }
          else
          {
            photoelec_delay_cnt = 0;
          }
          /*
          if (!intake_measure_data.limit_switch_value)
          {
            limit_switch_delay_cnt++;
            if (limit_switch_delay_cnt > 50)
            {
              pc_cmd = INTAKE_CMD_CLAMP;
              limit_switch_delay_cnt = 0;
            }
          }
          else
          {
            limit_switch_delay_cnt = 0;
          }
         */
        }break;
        
        case 4:
        {
          pc_cmd = INTAKE_CMD_CLAMP;
        }break;
        
        default:
        {
          pc_cmd = INTAKE_CMD_INIT;
        }
      }
    }
    break;
    
    case INTAKE_DEBUG:
    {
			intake.cur_state = INTAKE_INIT;
    }break;
    
    default:
    {
    }
    break;
  }
  
	last_intake_state = intake.cur_state;
	intake_set_state();
  
	if (last_intake_state != intake.cur_state)
  {
    intake_info_fdb();
	}
  pc_send_mesg.intake_information.intake_feedback = pc_msg;
	update_motion_feedforward(&intake_lift_mp);
	update_motion_feedforward(&intake_rot_mp);
	
	intake_rot_spd_calc(intake_rot_set_ecd(intake_rot_mp.ff_pos), intake.intake_rot_pos_fdb);
	//intake_rot_spd_calc(rot_set_pos, intake.intake_rot_pos_fdb); 
	//intake.intake_rot_cur  = pid_calc(&pid_intake_rot_spd, intake_measure_data.rotate_motor.speed_rpm, intake_rot_set_ecd(intake_rot_mp.ff_pos));
	intake.intake_rot_cur  = pid_calc(&pid_intake_rot_spd, intake_measure_data.rotate_motor.speed_rpm, intake.intake_rot_spd);
	//intake.intake_lift_cur = pid_calc(&pid_intake_lift_spd, intake_measure_data.lift_motor.speed_rpm, intake.intake_lift_spd);
	//intake.intake_lift_cur = pid_calc(&pid_intake_lift_spd, intake_measure_data.lift_motor.speed_rpm, 0);
	
	intake.intake_lift_cur =  pid_calc(&pid_intake_lift_spd, intake_measure_data.lift_motor.speed_rpm, pid_calc(&pid_intake_lift_pos, intake.intake_lift_pos_fdb, intake_lift_set_ecd(intake_lift_mp.ff_pos)));
	//intake.intake_lift_cur =  pid_calc(&pid_intake_lift_spd, intake_measure_data.lift_motor.speed_rpm, pid_calc(&pid_intake_lift_pos, intake.intake_lift_pos_fdb, intake_lift_set_ecd(motion_ff_pos)));
	//intake.intake_lift_cur = pid_calc(&pid_intake_lift_spd, intake_measure_data.lift_motor.speed_rpm, motion_ff_spd * 7.6 + pid_calc(&pid_intake_lift_pos, intake.intake_lift_pos_fdb, intake_lift_set_ecd(motion_ff_pos)));
	//intake.intake_lift_cur = pid_calc(&pid_intake_lift_pos, intake.intake_lift_pos_fdb, motion_ff_pos * 991.97) + pid_calc(&pid_intake_lift_spd, intake_measure_data.lift_motor.speed_rpm, motion_ff_spd * 7.261);
	
	
	memcpy(&glb_cur.intake_rot_cur, &intake.intake_rot_cur, sizeof(intake.intake_rot_cur));
	memcpy(&glb_cur.intake_lift_cur, &intake.intake_lift_cur, sizeof(intake.intake_lift_cur));
	
	// signal can_send_message
	osSignalSet(can_msg_send_task_t, INTAKE_MOTOR_MSG_SEND);
	
	intake_stack_surplus = uxTaskGetStackHighWaterMark(NULL);
}

void intake_param_init(void)
{
	memset(&intake, 0, sizeof(intake_t));
	intake.ctrl_mode = INTAKE_STOP;
	
	intake_measure_data.rot_initialized = 0;
	intake_measure_data.lift_initialized = 0;
	
	PID_struct_init(&pid_intake_rot_pos,  POSITION_PID,  2000, 500, 0.9, 0, 0.6);
	PID_struct_init(&pid_intake_rot_spd,  POSITION_PID, 15000, 500, 3.5, 0,   0);
	PID_struct_init(&pid_intake_lift_pos, POSITION_PID, 16384, 500, 0.5 , 0,   0.5);
	PID_struct_init(&pid_intake_lift_spd, POSITION_PID, 16384, 500, 3.5, 0,   0);
	
	intake.cur_state = INTAKE_INIT;
  
  //rotate motor param init
  intake_measure_data.rotate_motor.offset_ecd  = intake_measure_data.rotate_motor.ecd;
	intake_measure_data.rotate_motor.round_cnt   = 0;
	intake_measure_data.rotate_motor.total_ecd   = 0;
	intake_measure_data.rotate_motor.total_angle = 0;
	intake.intake_rot_spd = 0;
  
  //lift motor param init
  intake_measure_data.lift_motor.offset_ecd   = intake_measure_data.lift_motor.ecd;
	intake_measure_data.lift_motor.round_cnt    = 0;
	intake_measure_data.lift_motor.total_ecd    = 0;
	intake_measure_data.lift_motor.total_angle  = 0;
	intake.intake_lift_spd = 0;
	
  // initialize motion profile
	motion_profile_init(&intake_lift_mp, 100);
	motion_profile_init(&intake_rot_mp, 100);
  
  keep_up = 0;
	
	//photoelectric and supply param init, and photoelectric_value == 0 means be obscured
	intake_measure_data.photoelectric_value = 1;
  intake_measure_data.supply_check_value  = 1;
	
  photoelec_delay_cnt    = 0;
  close_delay_cnt        = 0;
  dump_delay_cnt         = 0;
  supply_check_delay_cnt = 0;
	lift_delay_cnt         = 0;
  
  CAMERA_GIMBAL = camera_mid_pos;
  
  supply_box_sum = 0;
  supply_fail    = 0;
	
	pc_msg = 0;
	
	from_clamp_down = 0;
}

/*
void intake_rot_init_handler(void)
{
	if (intake_measure_data.rot_initialized != 1)
	{
		 
	}
}
*/

/*
void intake_lift_init_handler(void)
{
	// initialize motion profile
	motion_count = 0;
	motion_max_acc = 0;
	motion_t1 = 0;
	motion_t2 = 0;
	motion_t3 = 0;
	
	
	if(intake_measure_data.lift_initialized != 1)
	{
		if(intake_measure_data.lift_limit_switch_value != 0)
		{
			intake.intake_lift_spd = -1;
		}
		else
		{
			intake_measure_data.lift_motor.offset_ecd   = intake_measure_data.lift_motor.ecd;
			intake_measure_data.lift_motor.round_cnt    = 0;
			intake_measure_data.lift_motor.total_ecd    = 0;
			intake_measure_data.lift_motor.total_angle  = 0;
			intake.intake_lift_spd = 0;
			intake_measure_data.lift_initialized = 1;
			shield_vision = 0;
		}
	}
}
*/

void intake_info_fdb()
{
  switch(last_intake_state)
  {
    case INTAKE_INIT:
    {
      pc_msg = 0;
    }break;
    
    case INTAKE_VISION:
    {
      pc_msg = 0;
    }break;
    
    case INTAKE_NORMAL_DOWN:
    {
      pc_msg = 0;
    }break;
    
    case INTAKE_NORMAL_UP:
    {
      pc_msg = 0;
    }break;
    
    case INTAKE_CLAMP_UP:
    {
      pc_msg = 1;
    }break;
    
    case INTAKE_CLAMP_DOWN:
    {
      pc_msg = 1;
    }break;
    
    case INTAKE_DUMP:
    {
      pc_msg = 2;
    }break;
    
    case INTAKE_RELEASE:
    {
      pc_msg = 4;
    }break;
    
    case INTAKE_VISION_UP:
    {
      pc_msg = 0;
    }break;
    
    case INTAKE_VISION_DOWN:
    {
      pc_msg = 0;
    }break;
    
    case INTAKE_END:
    {
      pc_msg = 4;
    }break;
  }
}

static void intake_rot_spd_calc(int32_t ref, int32_t fdb)
{
	intake.intake_rot_spd = pid_calc(&pid_intake_rot_pos, fdb, ref);
}

/*
static void intake_lift_spd_calc(int32_t ref, int32_t fdb)
{
	//intake.intake_lift_spd = pid_calc(&pid_intake_lift_pos, fdb, ref);
}
*/


double lift_init_pos = -310, lift_vision_pos = -310, lift_up_pos = -370, lift_down_pos = -10, lift_dump_pos = -350, lift_clamp_up_pos = -335;
int is_last_state_init = 0;
float intake_rot_init_angle = -190;

int32_t rot_145 = (int32_t)(-(145 + 15) * 8192 * M3508_RED_RATIO / 360);
int32_t rot_105 = (int32_t)(-(105 + 15) * 8192 * M3508_RED_RATIO / 360);
int32_t rot_90  = (int32_t)(-( 90 + 15) * 8192 * M3508_RED_RATIO / 360);
int32_t rot_0   = (int32_t)(-(  0 + 15) * 8192 * M3508_RED_RATIO / 360);


static void intake_set_state()
{
  switch(intake.cur_state)
  {
		case INTAKE_INIT:
		{
			is_last_state_init = 1;
			update_motion_set_pos(&intake_lift_mp, lift_init_pos, lift_slow_spd, lift_slow_acc, lift_slow_acc);
			
			close_clamp();
      CAMERA_GIMBAL = camera_mid_pos;
			
		  if(motion_profile_is_updatable(&intake_lift_mp))
			{
				rot_set_pos = rot_90;
				update_motion_set_pos(&intake_rot_mp, intake_rot_init_angle, rot_init_spd, rot_init_acc, rot_init_acc);
				
				//if (abs(rot_set_pos - intake_measure_data.rotate_motor.total_ecd) < 2000 )
				if(motion_profile_is_updatable(&intake_rot_mp))
				{
          if (pc_cmd == INTAKE_CMD_VISION)
          {
            intake.cur_state = INTAKE_VISION;
          } 
          else if (pc_cmd == INTAKE_CMD_VISION_LIFT_UP)
          {
            intake.cur_state = INTAKE_VISION_UP;
          }
          else if (pc_cmd == INTAKE_CMD_VISION_LIFT_DOWN)
          {
            intake.cur_state = INTAKE_VISION_DOWN;
          }
				}
			}
		}break;
    
		case INTAKE_VISION:
		{
			close_clamp();
			rot_set_pos = rot_90;
			if(is_last_state_init == 1)
			{
				update_motion_set_pos(&intake_rot_mp, -105, rot_init_spd, rot_init_acc, rot_init_acc);
			}
			else
			{
			  update_motion_set_pos(&intake_rot_mp, -105, rot_fast_spd, rot_fast_acc, rot_fast_acc);
			}
				
			if(motion_profile_is_updatable(&intake_rot_mp))
			{
        CAMERA_GIMBAL = camera_mid_pos;
			  update_motion_set_pos(&intake_lift_mp, lift_vision_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);
			
			
				//if (abs(rot_set_pos - intake_measure_data.rotate_motor.total_ecd) < 2000 && motion_profile_is_updatable(&intake_lift_mp))
				if (motion_profile_is_updatable(&intake_lift_mp))
				{
					if (pc_cmd == INTAKE_CMD_INIT)
					{
						intake.cur_state = INTAKE_INIT;
					}
					else if (pc_cmd == INTAKE_CMD_VISION_LIFT_UP)
					{
						intake.cur_state = INTAKE_VISION_UP;
					}
					else if (pc_cmd == INTAKE_CMD_VISION_LIFT_DOWN)
					{
						intake.cur_state = INTAKE_VISION_DOWN;
					}
				}
		  }
			is_last_state_init = 0;
		}break;
    
    case INTAKE_VISION_UP:
    {
      rot_set_pos = rot_90;
			
			if(is_last_state_init == 1)
			{
				update_motion_set_pos(&intake_rot_mp, -105, rot_init_spd, rot_init_acc, rot_init_acc);
			}
			else
			{
			  update_motion_set_pos(&intake_rot_mp, -105, rot_fast_spd, rot_fast_acc, rot_fast_acc);
      }
				
			CAMERA_GIMBAL = camera_up_pos;
      
      //if (abs(rot_set_pos - intake_measure_data.rotate_motor.total_ecd) < 2000)
			if(motion_profile_is_updatable(&intake_rot_mp))
      {
				close_clamp(); 
				update_motion_set_pos(&intake_lift_mp, lift_up_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);
        
				if(motion_profile_is_updatable(&intake_lift_mp)){
          if (pc_cmd == INTAKE_CMD_LIFT_UP)
          {
            intake.cur_state = INTAKE_NORMAL_UP;
          }
          else if (pc_cmd == INTAKE_CMD_INIT)
          {
            intake.cur_state = INTAKE_INIT;
          }
          else if (pc_cmd == INTAKE_CMD_VISION)
          {
            intake.cur_state = INTAKE_VISION;
          }
          else if (pc_cmd == INTAKE_CMD_VISION_LIFT_DOWN)
          {
            intake.cur_state = INTAKE_VISION_DOWN;
          }
        }
      }
			is_last_state_init = 0;
    }break;
    
    case INTAKE_VISION_DOWN:
    {
      rot_set_pos = rot_90;
			if(is_last_state_init == 1)
			{
		  	update_motion_set_pos(&intake_rot_mp, -105, rot_init_spd, rot_init_acc, rot_init_acc);
      }
			else
			{
				update_motion_set_pos(&intake_rot_mp, -105, rot_fast_spd, rot_fast_acc, rot_fast_acc);
			}
			CAMERA_GIMBAL = camera_down_pos;
      //intake_update_motion_profile(lift_down_pos);
      
     // if (abs(rot_set_pos - intake_measure_data.rotate_motor.total_ecd) < 2000)
			if(motion_profile_is_updatable(&intake_rot_mp))
      {
				close_clamp();
				update_motion_set_pos(&intake_lift_mp, lift_down_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);
				
        //if(intake_lift_has_reached())
        if(motion_profile_is_updatable(&intake_lift_mp))
				{
          if (pc_cmd == INTAKE_CMD_LIFT_DOWN)
          {
            intake.cur_state = INTAKE_NORMAL_DOWN;
          }
          else if (pc_cmd == INTAKE_CMD_INIT)
          {
            intake.cur_state = INTAKE_INIT;
          }
          else if (pc_cmd == INTAKE_CMD_VISION_LIFT_UP)
          {
            intake.cur_state = INTAKE_VISION_UP;
          }
          else if (pc_cmd == INTAKE_CMD_VISION)
          {
            intake.cur_state = INTAKE_VISION;
          }
        }
      }
			is_last_state_init = 0;
    }break;
    
		case INTAKE_NORMAL_UP:
		{
			open_clamp();

			update_motion_set_pos(&intake_lift_mp, lift_up_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);

      CAMERA_GIMBAL = camera_up_pos;

			
			//if (abs(rot_set_pos - intake_measure_data.rotate_motor.total_ecd) < 2000 && motion_profile_is_updatable(&intake_lift_mp))
			if(motion_profile_is_updatable(&intake_rot_mp) && motion_profile_is_updatable(&intake_lift_mp))
			{
				rot_set_pos = rot_0;
				update_motion_set_pos(&intake_rot_mp, -15, rot_fast_spd, rot_fast_acc, rot_fast_acc);
				if (pc_cmd == INTAKE_CMD_CLAMP)
				{
					intake.cur_state = INTAKE_CLAMP_UP;
					close_delay_cnt = 0;
				}
				else if (pc_cmd == INTAKE_CMD_VISION)
				{
					intake.cur_state = INTAKE_VISION;
				}
				else if (pc_cmd == INTAKE_CMD_VISION_LIFT_UP)
				{
					intake.cur_state = INTAKE_VISION_UP;
				}
				else if (pc_cmd == INTAKE_CMD_VISION_LIFT_DOWN)
				{
					intake.cur_state = INTAKE_VISION_DOWN;
				}
			}
		}break;
    
		case INTAKE_NORMAL_DOWN:
		{
			open_clamp();
			rot_set_pos = rot_0;
			update_motion_set_pos(&intake_rot_mp, -15, rot_fast_spd, rot_fast_acc, rot_fast_acc);
			
			update_motion_set_pos(&intake_lift_mp, lift_down_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);
			
      CAMERA_GIMBAL = camera_down_pos;
			
			//if (abs(rot_set_pos - intake_measure_data.rotate_motor.total_ecd) < 2000 && motion_profile_is_updatable(&intake_lift_mp))
			if(motion_profile_is_updatable(&intake_rot_mp) && motion_profile_is_updatable(&intake_lift_mp))
			{
				if (pc_cmd == INTAKE_CMD_CLAMP)
				{
					intake.cur_state = INTAKE_CLAMP_DOWN;
					close_delay_cnt = 0;
					lift_delay_cnt  = 0;
				}
				else if (pc_cmd == INTAKE_CMD_VISION)
				{
					intake.cur_state = INTAKE_VISION;
				}
				else if (pc_cmd == INTAKE_CMD_VISION_LIFT_UP)
				{
					intake.cur_state = INTAKE_VISION_UP;
				}
				else if (pc_cmd == INTAKE_CMD_VISION_LIFT_DOWN)
				{
					intake.cur_state = INTAKE_VISION_DOWN;
				}
			}
		}break;
    
		case INTAKE_CLAMP_UP:
		{
			if (from_clamp_down)
			{
				if (lift_delay_cnt > 30)
				{
					from_clamp_down = 0;
					lift_delay_cnt = 0;
				}
				else
				{
					lift_delay_cnt++;
				}
			}
			
			if (!from_clamp_down)
			{
				rot_set_pos = rot_0;
				update_motion_set_pos(&intake_rot_mp, -15, rot_fast_spd, rot_fast_acc, rot_fast_acc);
				//@TODO: remeber to change it;
				
				update_motion_set_pos(&intake_lift_mp, lift_clamp_up_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);
				
				CAMERA_GIMBAL = camera_up_pos;


				if (motion_profile_is_updatable(&intake_rot_mp))
				{
					close_clamp();
					
					if (close_delay_cnt > 20)
					{
						//if(intake_lift_has_reached())
						if(motion_profile_is_updatable(&intake_lift_mp))
						{
							intake.cur_state = INTAKE_DUMP;
							dump_delay_cnt = 0;
							supply_check_delay_cnt = 0;
							supply_success = 0;
						}
					}
					else
					{
						close_delay_cnt++;
					}
				}
			}
		}break;
    
		case INTAKE_CLAMP_DOWN:
		{
			close_clamp();
			rot_set_pos = rot_0;
			update_motion_set_pos(&intake_rot_mp, -15, rot_fast_spd, rot_fast_acc, rot_fast_acc);

			update_motion_set_pos(&intake_lift_mp, lift_down_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);

      CAMERA_GIMBAL = camera_down_pos;

			
			if (close_delay_cnt > 20)
			{
				from_clamp_down = 1;
				lift_delay_cnt = 0;
				intake.cur_state = INTAKE_CLAMP_UP;
			}
			else
			{
				close_delay_cnt++;
			}
		}break;
    
		case INTAKE_DUMP:
		{
			close_clamp();
			rot_set_pos = rot_105;
			update_motion_set_pos(&intake_rot_mp, -135, rot_fast_spd, rot_fast_acc, rot_fast_acc);
			
			update_motion_set_pos(&intake_lift_mp, lift_dump_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);
      
      /*
      if (!intake_measure_data.supply_check_value)
      {
        supply_check_delay_cnt++;
        if (supply_check_delay_cnt > 5)
        {
          supply_success = 1;
        }
      }
      else
      {
        supply_check_delay_cnt = 0;
      }
      */
      
			
			if (motion_profile_is_updatable(&intake_rot_mp) && motion_profile_is_updatable(&intake_lift_mp))
			{
				if (dump_delay_cnt > 70)
				{
          shoot.remain_bullets += 50;
					intake.cur_state = INTAKE_RELEASE;
				}
				else
				{
					dump_delay_cnt++;
				}
			}
		}break;
    
		case INTAKE_RELEASE:
		{
			rot_set_pos = rot_0;
			update_motion_set_pos(&intake_rot_mp, -15, rot_fast_spd, rot_fast_acc, rot_slow_acc);

			update_motion_set_pos(&intake_lift_mp, lift_clamp_up_pos, lift_fast_spd, lift_fast_acc, lift_fast_acc);
      
      /*
      if (!intake_measure_data.supply_check_value)
      {
        supply_check_delay_cnt++;
        if (supply_check_delay_cnt > 5)
        {
          supply_success = 1;
        }
      }
      else
      {
        supply_check_delay_cnt = 0;
      }
      */
			
			// drop box mid-air
			/*
			if(intake_angle_measure > -50)
			{
				open_clamp();
			}
			else
			{
			  close_clamp();
		  }
			*/
			
			//if(abs(rot_0 - intake_measure_data.rotate_motor.total_ecd) < 2000 && motion_profile_is_updatable(&intake_lift_mp))
			if(intake_angle_measure > -80)
			{
				open_clamp();
			}
			else
			{
				close_clamp();
			}
			
			if(motion_profile_is_updatable(&intake_rot_mp) && motion_profile_is_updatable(&intake_lift_mp))
			{
				//open_clamp();
				
        /*
        if (!supply_success)
        {
          supply_fail = 1;
        }
        else
        {
          supply_box_sum++;
          supply_success = 0;
        }
        */
        
				release_delay_cnt = 0;
				intake.cur_state = INTAKE_END;
			}
			//else
			//{
				//close_clamp();
			//}
		}break;
    
    case INTAKE_END:
    {
      if (release_delay_cnt > 40)
      {
        if (pc_cmd == INTAKE_CMD_VISION_LIFT_UP)
        {
          intake.cur_state = INTAKE_VISION_UP;
        }
        else if (pc_cmd == INTAKE_CMD_VISION_LIFT_DOWN)
        {
          intake.cur_state = INTAKE_VISION_DOWN;
        }
        else if (pc_cmd == INTAKE_CMD_VISION)
        {
          intake.cur_state = INTAKE_VISION;
        }
      }
      else
      {
        release_delay_cnt++;
      }
    }break;
	}
}

float intake_lift_get_pos()
{
	return intake_measure_data.lift_motor.total_ecd / 1037.65;
}

int32_t intake_lift_set_ecd(float pos)
{
	return (int32_t)(pos * 1037.65);
}

int32_t intake_rot_set_ecd(float angle)
{
	return (int32_t)(angle * 432.36);
}

float intake_rot_get_angle()
{
	return intake_measure_data.rotate_motor.total_angle / 19;
}
