#ifndef INTAKE_TASK_H
#define INTAKE_TASK_H

#include "stm32f4xx_hal.h"
#include "infantry_info.h"
#include "bsp_can.h"
#include "bsp_io.h"



#define INTAKE_PERIOD 10
#define INTAKE_FREQ (1000 / INTAKE_PERIOD)

#define M3508_RED_RATIO 19 //motor 3508's reduction ratio is 19:1

#define INTAKE_ROT_INIT_POS  (2048 * M3508_RED_RATIO)  //90deg
#define INTAKE_LIFT_INIT_POS 0


#define INTAKE_ROT_DOWN_POS    0
#define INTAKE_ROT_PICK_UP_POS (796 * M3508_RED_RATIO)  //35deg
#define INTAKE_LIFT_UP_POS     (796 * M3508_RED_RATIO)
#define INTAKE_LIFT_DOWN_POS   0
#define INTAKE_VISION_POS      (796 * M3508_RED_RATIO)

#define CAMERA_GIMBAL TIM1->CCR3



typedef enum
{
	INTAKE_RELAX 	= 0,
	INTAKE_STOP 	= 1,
	INTAKE_MANUAL = 2,
	INTAKE_AUTO 	= 3,
  INTAKE_DEBUG  = 4,
} intake_ctrl_e;


typedef enum
{
	INTAKE_ROT_INIT= 0, // initial state, arm is vertical   and paw is open 
	INTAKE_ROT_NORMAL  = 1, // normal state, arm is horizontal and paw is open
	INTAKE_ROT_CLAMP   = 2, //
	INTAKE_ROT_PICK_UP = 3,
  INTAKE_ROT_FINISH  = 4,
} intake_rot_state_e;

typedef enum
{
  INTAKE_LIFT_KEEP_UP   = 0,
  INTAKE_LIFT_KEEP_DOWN = 1,
  INTAKE_LIFT_GO_UP     = 2,
  INTAKE_LIFT_GO_DOWN   = 3,
} intake_lift_state_e;

typedef enum
{
  INTAKE_INIT        = 0,
  INTAKE_VISION      = 1,
  INTAKE_NORMAL_DOWN = 2,
  INTAKE_NORMAL_UP   = 3,
	INTAKE_CLAMP_UP    = 4,
	INTAKE_CLAMP_DOWN  = 5,
	INTAKE_DUMP        = 6,
	INTAKE_RELEASE     = 7,
  INTAKE_VISION_UP   = 8,
  INTAKE_VISION_DOWN = 9,
  INTAKE_END         = 10,
} intake_state_e;

typedef enum
{
  INTAKE_CMD_VISION           = 0,
	INTAKE_CMD_LIFT_UP          = 1,
	INTAKE_CMD_LIFT_DOWN        = 2,
	INTAKE_CMD_CLAMP            = 3,
  INTAKE_CMD_VISION_LIFT_UP   = 4,
  INTAKE_CMD_VISION_LIFT_DOWN = 5,
  INTAKE_CMD_INIT             = 6,
} intake_cmd_e;

typedef struct
{
	moto_measure_t rotate_motor;
	moto_measure_t lift_motor;
	
	uint8_t solenoid_set;
	uint8_t photoelectric_value;
	uint8_t supply_check_value;
	uint8_t clamp_reed_switch_value;
	uint8_t lift_limit_switch_value;

  uint8_t rot_initialized;
	uint8_t lift_initialized;
} intake_measure_t;

typedef struct
{
	float					intake_rot_spd;
	float					intake_lift_spd;
	
	int16_t				intake_rot_cur;
	int16_t				intake_lift_cur;
	
	int32_t				intake_rot_pos_ref;
	int32_t				intake_rot_pos_fdb;
	int32_t				intake_lift_pos_ref;
	int32_t				intake_lift_pos_fdb;
	
	intake_ctrl_e ctrl_mode;
  
	intake_rot_state_e cur_rot_state;
	intake_rot_state_e set_rot_state;
  
  intake_lift_state_e cur_lift_state;
  intake_lift_state_e set_lift_state;
	
	intake_state_e cur_state;
} intake_t;

extern intake_t intake;
extern intake_measure_t intake_measure_data;

void intake_task(void const *argu);

void intake_param_init(void);

//void intake_rot_init_handler(void);

//void intake_lift_init_handler(void);

void intake_info_fdb(void);

static void intake_rot_spd_calc(int32_t ref, int32_t fdb);

//static void intake_lift_spd_calc(int32_t ref, int32_t fdb);

static void intake_set_state(void);

float intake_lift_get_pos(void);

float intake_rot_get_angle(void);

int32_t intake_lift_set_ecd(float pos);

int32_t intake_rot_set_ecd(float angle);
#endif
