#include "motion_profile.h"

void motion_profile_init(motion_t* mp, int set_motion_update_freq)
{
	mp->motion_update_freq = set_motion_update_freq;
	mp->motion_set_pos = 0;
	mp->motion_start_pos = 0;
}


void update_motion_set_pos(motion_t* mp, float set_pos, 
	float set_max_spd, float set_ramp_up_acc, float set_ramp_down_acc)
{
	//motion_profile_is_updatable(mp) &&
	if(set_pos != mp->motion_set_pos)
	{
		mp->max_spd = set_max_spd;
	  mp->ramp_up_acc = set_ramp_up_acc;
	  mp->ramp_down_acc = set_ramp_down_acc;
		mp->motion_start_pos = mp->ff_pos;
		mp->motion_set_pos = set_pos;
		
		if(mp->motion_set_pos > mp->motion_start_pos)
		{
			mp->motion_dir = 1;
		}
		else
		{
			mp->motion_dir = -1;
		}
		
		float displacement = fabs(mp->motion_set_pos - mp->motion_start_pos);
		float motion_max_spd_power_2 = mp->max_spd * mp->max_spd / 2;
		
		float shortest_trap_distance = motion_max_spd_power_2 / mp->ramp_up_acc + motion_max_spd_power_2 / mp->ramp_down_acc;
		//mp->ramp_up_area = motion_max_spd_power_2 / mp->ramp_up_acc;
		//mp->ramp_down_area = motion_max_spd_power_2 / mp->ramp_down_acc;
		//float shortest_trap_distance = mp->ramp_up_area + mp->ramp_down_area; 
		
		// triagular_velocity_mode
		if(displacement <= shortest_trap_distance)
		{
			float temp_max_spd = mp->max_spd * sqrt(displacement / shortest_trap_distance);
			float tmep_max_spd_power_2 = temp_max_spd * temp_max_spd / 2;
			
			mp->ramp_up_area = displacement * mp->ramp_down_acc / (mp->ramp_up_acc + mp->ramp_down_acc); //tmep_max_spd_power_2 / mp->ramp_up_acc;
			mp->ramp_down_area = displacement - mp->ramp_up_area;
			mp->max_vel_area = 0;
			
			mp->motion_t1 = mp->motion_update_freq * temp_max_spd / mp->ramp_up_acc;
			mp->motion_t2 = mp->motion_t1;
			mp->motion_t3 = mp->motion_t2 + mp->motion_update_freq * temp_max_spd / mp->ramp_down_acc;
		}
		// trapezoidal_velocity_mode
		else
		{
			mp->ramp_up_area = motion_max_spd_power_2 / mp->ramp_up_acc;
			mp->ramp_down_area = motion_max_spd_power_2 / mp->ramp_down_acc;
			mp->max_vel_area = displacement - mp->ramp_up_area -	mp->ramp_down_area;
			
			mp->motion_t1 = mp->motion_update_freq * mp->max_spd / mp->ramp_up_acc;
			mp->motion_t2 = mp->motion_t1 + mp->motion_update_freq * (displacement - shortest_trap_distance) / mp->max_spd; 
			mp->motion_t3 = mp->motion_t2 + mp->motion_update_freq * mp->max_spd / mp->ramp_down_acc;
		}
		
		mp->motion_counter = 0;
	}
}

void update_motion_feedforward(motion_t* mp)
{
	float pos_time = 0;
	// update velocity and position feedforward
	// velocity ramp up
	if(mp->motion_counter <= mp->motion_t1)
	{
		mp->ff_vel = mp->motion_dir * mp->ramp_up_acc * mp->motion_counter / mp->motion_update_freq;
		//mp->ff_pos += mp->ff_vel /  mp->motion_update_freq;
		pos_time = (float)mp->motion_counter / mp->motion_t1;
		mp->ff_pos = mp->motion_start_pos + mp->motion_dir * mp->ramp_up_area * pos_time * pos_time;
	}
	// maximum velocity
	else if(mp->motion_counter <= mp->motion_t2)
	{
		mp->ff_vel = mp->motion_dir * mp->max_spd;
		mp->ff_pos += mp->ff_vel / mp->motion_update_freq;	
	}
	// velocity ramp down
	else if(mp->motion_counter <= mp->motion_t3)
	{
		mp->ff_vel = mp->motion_dir * mp->ramp_down_acc * (mp->motion_t3 - mp->motion_counter) / mp->motion_update_freq;
		//mp->ff_pos += mp->ff_vel / mp->motion_update_freq;
		pos_time = (float)(mp->motion_t3 - mp->motion_counter) / (mp->motion_t3 - mp->motion_t2);
	  mp->ff_pos = mp->motion_set_pos - mp->motion_dir * mp->ramp_down_area * pos_time * pos_time;
	}
	// zero velocity
	else
	{
		mp->ff_vel = 0;
		mp->ff_pos = mp->motion_set_pos;
	}
	
	mp->motion_counter++;
}

int motion_profile_is_updatable(motion_t* mp)
{
	if(mp->motion_counter >= mp->motion_t3)
		return 1;
	else
		return 0;
}

motion_t intake_lift_mp = {0};
motion_t intake_rot_mp = {0};
