#ifndef MOTION_PROFILE_H_
#define MOTION_PROFILE_H_

#include "stm32f4xx_hal.h"
#include "math.h"

typedef struct motion_t
{
	int motion_update_freq;
	
	float motion_start_pos;
	float motion_set_pos;
	
	int motion_dir;
	float ff_vel;
	float ff_pos;
	
	uint32_t motion_counter;
	uint32_t motion_t1;
	uint32_t motion_t2;
	uint32_t motion_t3;
	
	float max_spd;
	float ramp_up_acc;
	float ramp_down_acc;
	
	float ramp_up_area;
	float ramp_down_area;
	float max_vel_area;
} motion_t;

void motion_profile_init(motion_t* mp, int set_motion_update_freq);

void update_motion_set_pos(motion_t* mp, float set_pos, 
	float set_max_spd, float set_ramp_up_acc, float set_ramp_down_acc);

void update_motion_feedforward(motion_t* mp);

int motion_profile_is_updatable(motion_t* mp);

extern motion_t intake_lift_mp;
extern motion_t intake_rot_mp;

#endif
